@@ -38,6 +38,9 @@ from autoarray.structures.arrays.kernel_2d import Kernel2D
 from autoarray.structures.visibilities import Visibilities
 from autoarray.structures.visibilities import VisibilitiesNoiseMap
 
+from autogalaxy import cosmology as cosmo
+from autogalaxy.gui.clicker import Clicker
+from autogalaxy.gui.scribbler import Scribbler
 from autogalaxy.galaxy.galaxy import Galaxy, HyperGalaxy, Redshift
 from autogalaxy.analysis.clump_model import ClumpModel
 from autogalaxy.analysis.clump_model import ClumpModelDisabled
@@ -55,18 +58,26 @@ from autogalaxy.profiles import (
 )
 from autogalaxy.profiles.light_profiles import light_profiles_init as lp_init
 from autogalaxy.profiles.light_profiles import light_profiles_snr as lp_snr
+from autogalaxy.profiles.light_profiles import light_profiles_linear as lp_linear
+from autogalaxy.profiles.light_profiles import light_profiles_operated as lp_operated
+from autogalaxy.profiles.light_profiles import (
+    light_profiles_linear_operated as lp_linear_operated,
+)
 from autogalaxy.operate.image import OperateImage
 from autogalaxy.operate.deflections import OperateDeflections
 from autogalaxy.quantity.dataset_quantity import DatasetQuantity
 from autogalaxy import convert
 
 from . import plot
-from .lens.model import aggregator as agg
+from . import aggregator as agg
 from .lens import subhalo
-from .lens.model.settings import SettingsLens
+from .analysis.settings import SettingsLens
 from .lens.ray_tracing import Tracer
-from .lens.model.preloads import Preloads
-from .lens.model.setup import SetupHyper
+from .lens.to_inversion import TracerToInversion
+from .analysis.positions import PositionsLHResample
+from .analysis.positions import PositionsLHPenalty
+from .analysis.preloads import Preloads
+from .analysis.setup import SetupHyper
 from .imaging.imaging import SimulatorImaging
 from .imaging.fit_imaging import FitImaging
 from .imaging.model.analysis import AnalysisImaging
@@ -85,6 +96,7 @@ from .point.model.analysis import AnalysisPoint
 from .point.point_solver import PointSolver
 from .quantity.fit_quantity import FitQuantity
 from .quantity.model.analysis import AnalysisQuantity
+from . import exc
 from . import mock as m
 from . import util
 
@@ -92,4 +104,4 @@ from autoconf import conf
 
 conf.instance.register(__file__)
 
-__version__ = "2022.03.30.1"
+__version__ = "2022.07.11.1"
