@@ -1,22 +1,25 @@
+from typing import Optional
+
 import autofit as af
-from autogalaxy.analysis import setup
-from autogalaxy.profiles import mass_profiles as mp, light_and_mass_profiles as lmp
-from autogalaxy.hyper import hyper_data as hd
-from autogalaxy.galaxy import galaxy as g
-
-from typing import Tuple, Union, Optional
+import autogalaxy as ag
 
 
-class SetupHyper(setup.SetupHyper):
+class SetupHyper(ag.SetupHyper):
     def __init__(
         self,
         hyper_galaxies_lens: bool = False,
         hyper_galaxies_source: bool = False,
-        hyper_image_sky: Optional[type(hd.HyperImageSky)] = None,
-        hyper_background_noise: Optional[type(hd.HyperBackgroundNoise)] = None,
+        hyper_image_sky: Optional[type(ag.hyper_data.HyperImageSky)] = None,
+        hyper_background_noise: Optional[
+            type(ag.hyper_data.HyperBackgroundNoise)
+        ] = None,
         hyper_fixed_after_source: bool = False,
-        search_cls: Optional[af.NonLinearSearch] = None,
-        search_dict: Optional[dict] = None,
+        search_inversion_cls: Optional[af.NonLinearSearch] = None,
+        search_noise_cls: Optional[af.NonLinearSearch] = None,
+        search_bc_cls: Optional[af.NonLinearSearch] = None,
+        search_pixelized_dict: Optional[dict] = None,
+        search_noise_dict: Optional[dict] = None,
+        search_bc_dict: Optional[dict] = None,
     ):
         """
         The hyper setup of a pipeline, which controls how hyper-features in PyAutoLens template pipelines run,
@@ -40,9 +43,9 @@ class SetupHyper(setup.SetupHyper):
             If `True`, the hyper parameters are fixed and not updated after a desnated pipeline in the analysis. For
             the `SLaM` pipelines this is after the `SourcePipeline`. This allow Bayesian model comparison to be
             performed objected between later searchs in a pipeline.
-        search_cls
+        search_inversion_cls
             The non-linear search used by every hyper model-fit search.
-        search_dict
+        search_pixelized_dict
             The dictionary of search options for the hyper model-fit searches.
         """
         hyper_galaxies = hyper_galaxies_lens or hyper_galaxies_source
@@ -51,8 +54,12 @@ class SetupHyper(setup.SetupHyper):
             hyper_galaxies=hyper_galaxies,
             hyper_image_sky=hyper_image_sky,
             hyper_background_noise=hyper_background_noise,
-            search_cls=search_cls,
-            search_dict=search_dict,
+            search_inversion_cls=search_inversion_cls,
+            search_noise_cls=search_noise_cls,
+            search_bc_cls=search_bc_cls,
+            search_pixelized_dict=search_pixelized_dict,
+            search_noise_dict=search_noise_dict,
+            search_bc_dict=search_bc_dict,
         )
 
         self.hyper_galaxies_lens = hyper_galaxies_lens
@@ -69,9 +76,7 @@ class SetupHyper(setup.SetupHyper):
 
         self.hyper_fixed_after_source = hyper_fixed_after_source
 
-    def hyper_galaxy_lens_from_result(
-        self, result: af.Result, noise_factor_is_model=False
-    ):
+    def hyper_galaxy_lens_from(self, result: af.Result, noise_factor_is_model=False):
         """
         Returns the `HyperGalaxy` `Model` from a previous pipeline or search of the lens galaxy in a template
         PyAutoLens pipeline.
@@ -86,9 +91,9 @@ class SetupHyper(setup.SetupHyper):
 
         Parameters
         ----------
-        index : int
+        index
             The index of the previous search the `HyperGalaxy` `Model` is passed from.
-        noise_factor_is_model : bool
+        noise_factor_is_model
             If `True` the `noise_factor` of the `HyperGalaxy` is passed as a `model`, else it is passed as an
             `instance`.
 
@@ -102,20 +107,19 @@ class SetupHyper(setup.SetupHyper):
             return None
 
         if hasattr(result, "hyper"):
-            return self.hyper_galaxy_from_galaxy_model_and_instance(
+            return self.hyper_galaxy_via_galaxy_model_from(
                 galaxy_model=result.hyper.model.galaxies.lens,
                 galaxy_instance=result.hyper.instance.galaxies.lens,
                 noise_factor_is_model=noise_factor_is_model,
             )
 
-        return self.hyper_galaxy_from_galaxy_model_and_instance(
+        return self.hyper_galaxy_via_galaxy_model_from(
             galaxy_model=result.model.galaxies.lens,
             galaxy_instance=result.instance.galaxies.lens,
+            noise_factor_is_model=noise_factor_is_model,
         )
 
-    def hyper_galaxy_source_from_result(
-        self, result: af.Result, noise_factor_is_model=False
-    ):
+    def hyper_galaxy_source_from(self, result: af.Result, noise_factor_is_model=False):
         """
         Returns the `HyperGalaxy` `Model` from a previous pipeline or search of the source galaxy in a template
         PyAutosource pipeline.
@@ -130,9 +134,9 @@ class SetupHyper(setup.SetupHyper):
 
         Parameters
         ----------
-        index : int
+        index
             The index of the previous search the `HyperGalaxy` `Model` is passed from.
-        noise_factor_is_model : bool
+        noise_factor_is_model
             If `True` the `noise_factor` of the `HyperGalaxy` is passed as a `model`, else it is passed as an
             `instance`.
 
@@ -146,22 +150,23 @@ class SetupHyper(setup.SetupHyper):
             return None
 
         if hasattr(result, "hyper"):
-            return self.hyper_galaxy_from_galaxy_model_and_instance(
+            return self.hyper_galaxy_via_galaxy_model_from(
                 galaxy_model=result.hyper.model.galaxies.source,
                 galaxy_instance=result.hyper.instance.galaxies.source,
                 noise_factor_is_model=noise_factor_is_model,
             )
 
-        return self.hyper_galaxy_from_galaxy_model_and_instance(
+        return self.hyper_galaxy_via_galaxy_model_from(
             galaxy_model=result.model.galaxies.source,
             galaxy_instance=result.instance.galaxies.source,
+            noise_factor_is_model=noise_factor_is_model,
         )
 
-    def hyper_galaxy_from_galaxy_model_and_instance(
+    def hyper_galaxy_via_galaxy_model_from(
         self, galaxy_model, galaxy_instance, noise_factor_is_model=False
     ):
 
-        hyper_galaxy = af.Model(g.HyperGalaxy)
+        hyper_galaxy = af.Model(ag.HyperGalaxy)
 
         if galaxy_model.hyper_galaxy is None:
             return None
