@@ -7,20 +7,23 @@ from autoconf import cached_property
 import autoarray as aa
 import autogalaxy as ag
 
-from autolens.lens.model.preloads import Preloads
+from autogalaxy.abstract_fit import AbstractFit
+
+from autolens.analysis.preloads import Preloads
+from autolens.lens.ray_tracing import Tracer
 
 
-class FitImaging(aa.FitImaging):
+class FitImaging(aa.FitImaging, AbstractFit):
     def __init__(
         self,
-        dataset,
-        tracer,
-        hyper_image_sky=None,
-        hyper_background_noise=None,
-        use_hyper_scaling=True,
-        settings_pixelization=aa.SettingsPixelization(),
-        settings_inversion=aa.SettingsInversion(),
-        preloads=Preloads(),
+        dataset: aa.Imaging,
+        tracer: Tracer,
+        hyper_image_sky: Optional[ag.hyper_data.HyperImageSky] = None,
+        hyper_background_noise: Optional[ag.hyper_data.HyperBackgroundNoise] = None,
+        use_hyper_scaling: bool = True,
+        settings_pixelization: aa.SettingsPixelization = aa.SettingsPixelization(),
+        settings_inversion: aa.SettingsInversion = aa.SettingsInversion(),
+        preloads: Preloads = Preloads(),
         profiling_dict: Optional[Dict] = None,
     ):
         """
@@ -34,6 +37,9 @@ class FitImaging(aa.FitImaging):
         """
 
         super().__init__(dataset=dataset, profiling_dict=profiling_dict)
+        AbstractFit.__init__(
+            self=self, model_obj=tracer, settings_inversion=settings_inversion
+        )
 
         self.tracer = tracer
 
@@ -87,7 +93,7 @@ class FitImaging(aa.FitImaging):
 
         if self.preloads.blurred_image is None:
 
-            return self.tracer.blurred_image_2d_via_convolver_from(
+            return self.tracer.blurred_image_2d_from(
                 grid=self.dataset.grid,
                 convolver=self.dataset.convolver,
                 blurring_grid=self.dataset.blurring_grid,
@@ -109,14 +115,13 @@ class FitImaging(aa.FitImaging):
 
         The image passed to this function is the dataset's image with all light profile images of the tracer subtracted.
         """
-        if self.tracer.has_pixelization:
+        if self.perform_inversion:
 
-            return self.tracer.inversion_imaging_from(
-                grid=self.dataset.grid_inversion,
+            return self.tracer.to_inversion.inversion_imaging_from(
+                dataset=self.dataset,
                 image=self.profile_subtracted_image,
                 noise_map=self.noise_map,
-                convolver=self.dataset.convolver,
-                w_tilde=self.dataset.w_tilde,
+                w_tilde=self.w_tilde,
                 settings_pixelization=self.settings_pixelization,
                 settings_inversion=self.settings_inversion,
                 preloads=self.preloads,
@@ -133,7 +138,9 @@ class FitImaging(aa.FitImaging):
         If a inversion is included it is the sum of this sum and the inversion's reconstruction of the image.
         """
 
-        if self.tracer.has_pixelization:
+        if self.tracer.has(cls=aa.pix.Pixelization) or self.tracer.has(
+            cls=ag.lp_linear.LightProfileLinear
+        ):
 
             return self.blurred_image + self.inversion.mapped_reconstructed_data
 
@@ -144,44 +151,39 @@ class FitImaging(aa.FitImaging):
         return self.imaging.grid
 
     @property
-    def galaxy_model_image_dict(self) -> {ag.Galaxy: np.ndarray}:
+    def galaxy_model_image_dict(self) -> Dict[ag.Galaxy, np.ndarray]:
         """
         A dictionary associating galaxies with their corresponding model images
         """
-        galaxy_model_image_dict = self.tracer.galaxy_blurred_image_2d_dict_via_convolver_from(
+        galaxy_blurred_image_2d_dict = self.tracer.galaxy_blurred_image_2d_dict_from(
             grid=self.grid,
             convolver=self.imaging.convolver,
             blurring_grid=self.imaging.blurring_grid,
         )
 
-        # TODO : Extend to multiple inversioons across Planes
+        galaxy_linear_obj_image_dict = self.galaxy_linear_obj_data_dict_from(
+            use_image=True
+        )
 
-        for plane_index in self.tracer.plane_indexes_with_pixelizations:
-
-            galaxy_model_image_dict.update(
-                {
-                    self.tracer.planes[plane_index].galaxies[
-                        0
-                    ]: self.inversion.mapped_reconstructed_image
-                }
-            )
-
-        return galaxy_model_image_dict
+        return {**galaxy_blurred_image_2d_dict, **galaxy_linear_obj_image_dict}
 
     @property
     def model_images_of_planes_list(self):
 
-        model_images_of_planes_list = self.tracer.blurred_image_2d_list_via_psf_from(
-            grid=self.grid,
-            psf=self.imaging.psf,
-            blurring_grid=self.imaging.blurring_grid,
-        )
+        galaxy_model_image_dict = self.galaxy_model_image_dict
 
-        for plane_index in self.tracer.plane_indexes_with_pixelizations:
+        model_images_of_planes_list = [
+            aa.Array2D.manual_mask(
+                array=np.zeros(self.dataset.grid.shape_slim), mask=self.dataset.mask
+            )
+            for i in range(self.tracer.total_planes)
+        ]
 
-            model_images_of_planes_list[
-                plane_index
-            ] += self.inversion.mapped_reconstructed_image
+        for plane_index, plane in enumerate(self.tracer.planes):
+            for galaxy in plane.galaxies:
+                model_images_of_planes_list[plane_index] += galaxy_model_image_dict[
+                    galaxy
+                ]
 
         return model_images_of_planes_list
 
@@ -208,20 +210,16 @@ class FitImaging(aa.FitImaging):
 
     @property
     def unmasked_blurred_image(self):
-        return self.tracer.unmasked_blurred_image_2d_via_psf_from(
+        return self.tracer.unmasked_blurred_image_2d_from(
             grid=self.grid, psf=self.imaging.psf
         )
 
     @property
     def unmasked_blurred_image_of_planes_list(self):
-        return self.tracer.unmasked_blurred_image_2d_list_via_psf_from(
+        return self.tracer.unmasked_blurred_image_2d_list_from(
             grid=self.grid, psf=self.imaging.psf
         )
 
-    @property
-    def total_mappers(self):
-        return len(list(filter(None, self.tracer.regularization_pg_list)))
-
     def refit_with_new_preloads(self, preloads, settings_inversion=None):
 
         profiling_dict = {} if self.profiling_dict is not None else None
