@@ -8,13 +8,12 @@ from os import path
 
 from autoconf import conf
 import autofit as af
-import autoarray as aa
 import autogalaxy as ag
 
 from autoarray.exc import PixelizationException
 
-from autolens.lens.model.analysis import AnalysisDataset
-from autolens.lens.model.preloads import Preloads
+from autolens.analysis.analysis import AnalysisDataset
+from autolens.analysis.preloads import Preloads
 from autolens.lens.ray_tracing import Tracer
 from autolens.imaging.model.result import ResultImaging
 from autolens.imaging.model.visualizer import VisualizerImaging
@@ -37,9 +36,14 @@ class AnalysisImaging(AnalysisDataset):
         PyAutoFit calls this function immediately before the non-linear search begins, therefore it can be used to
         perform tasks using the final model parameterization.
 
-        This function checks that the hyper-dataset is consistent with previous hyper-datasets if the model-fit is
-        being resumed from a previous run, and it visualizes objects which do not change throughout the model fit
-        like the dataset.
+        This function:
+
+        - Checks that the hyper-dataset is consistent with previous hyper-datasets if the model-fit is being
+        resumed from a previous run.
+
+        - Checks the model and raises exceptions if certain critieria are not met.
+
+        Once inherited from it also visualizes objects which do not change throughout the model fit like the dataset.
 
         Parameters
         ----------
@@ -50,7 +54,7 @@ class AnalysisImaging(AnalysisDataset):
             The PyAutoFit model object, which includes model components representing the galaxies that are fitted to
             the imaging data.
         """
-        self.check_and_replace_hyper_images(paths=paths)
+        super().modify_before_fit(paths=paths, model=model)
 
         if not paths.is_complete:
 
@@ -58,15 +62,17 @@ class AnalysisImaging(AnalysisDataset):
 
             visualizer.visualize_imaging(imaging=self.imaging)
 
+            if self.positions_likelihood is not None:
+                visualizer.visualize_image_with_positions(
+                    image=self.imaging.image,
+                    positions=self.positions_likelihood.positions,
+                )
+
             visualizer.visualize_hyper_images(
                 hyper_galaxy_image_path_dict=self.hyper_galaxy_image_path_dict,
                 hyper_model_image=self.hyper_model_image,
             )
 
-            logger.info(
-                "PRELOADS - Setting up preloads, may take a few minutes for fits using an inversion."
-            )
-
             self.set_preloads(paths=paths, model=model)
 
         return self
@@ -110,6 +116,18 @@ class AnalysisImaging(AnalysisDataset):
             The log likelihood indicating how well this model instance fitted the imaging data.
         """
 
+        try:
+            log_likelihood_positions_overwrite = self.log_likelihood_positions_overwrite_from(
+                instance=instance
+            )
+            if log_likelihood_positions_overwrite is not None:
+                return log_likelihood_positions_overwrite
+        except Exception as e:
+            raise e
+
+        if log_likelihood_positions_overwrite is not None:
+            return log_likelihood_positions_overwrite
+
         try:
             return self.fit_imaging_via_instance_from(instance=instance).figure_of_merit
         except (
@@ -118,6 +136,7 @@ class AnalysisImaging(AnalysisDataset):
             exc.InversionException,
             exc.GridException,
             ValueError,
+            TypeError,
             np.linalg.LinAlgError,
             OverflowError,
         ) as e:
@@ -128,7 +147,6 @@ class AnalysisImaging(AnalysisDataset):
         instance: af.ModelInstance,
         use_hyper_scaling: bool = True,
         preload_overwrite: Optional[Preloads] = None,
-        check_positions: bool = True,
         profiling_dict: Optional[Dict] = None,
     ) -> FitImaging:
         """
@@ -163,11 +181,6 @@ class AnalysisImaging(AnalysisDataset):
             instance=instance, profiling_dict=profiling_dict
         )
 
-        if check_positions:
-            self.settings_lens.check_positions_trace_within_threshold_via_tracer(
-                tracer=tracer, positions=self.positions
-            )
-
         hyper_image_sky = self.hyper_image_sky_via_instance_from(instance=instance)
 
         hyper_background_noise = self.hyper_background_noise_via_instance_from(
@@ -289,7 +302,7 @@ class AnalysisImaging(AnalysisDataset):
 
         info_dict["image_pixels"] = self.imaging.grid.sub_shape_slim
         info_dict["sub_size_light_profiles"] = self.imaging.grid.sub_size
-        info_dict["sub_size_inversion"] = self.imaging.grid_inversion.sub_size
+        info_dict["sub_size_pixelized"] = self.imaging.grid_pixelized.sub_size
         info_dict["psf_shape_2d"] = self.imaging.psf.shape_native
 
         if fit.inversion is not None:
@@ -343,10 +356,12 @@ class AnalysisImaging(AnalysisDataset):
         instance = self.instance_with_associated_hyper_images_from(instance=instance)
         tracer = self.tracer_via_instance_from(instance=instance)
 
-        if not tracer.has_pixelization:
+        if not tracer.has(cls=ag.pix.Pixelization):
             return
 
-        if not any([pix.is_stochastic for pix in tracer.pixelization_list]):
+        if not any(
+            [pix.is_stochastic for pix in tracer.cls_list_from(cls=ag.pix.Pixelization)]
+        ):
             return
 
         hyper_image_sky = self.hyper_image_sky_via_instance_from(instance=instance)
@@ -425,12 +440,21 @@ class AnalysisImaging(AnalysisDataset):
             If True the visualization is being performed midway through the non-linear search before it is finished,
             which may change which images are output.
         """
+
         instance = self.instance_with_associated_hyper_images_from(instance=instance)
 
-        try:
-            fit = self.fit_imaging_via_instance_from(instance=instance)
-        except exc.RayTracingException:
-            return
+        fit = self.fit_imaging_via_instance_from(instance=instance)
+
+        if self.positions_likelihood is not None:
+            self.positions_likelihood.output_positions_info(
+                output_path=paths.output_path, tracer=fit.tracer
+            )
+
+        if fit.inversion is not None:
+            try:
+                fit.inversion.reconstruction
+            except exc.InversionException:
+                return
 
         visualizer = VisualizerImaging(visualize_path=paths.image_path)
 
@@ -461,39 +485,13 @@ class AnalysisImaging(AnalysisDataset):
                 fit=fit, during_analysis=during_analysis, subfolders="fit_no_hyper"
             )
 
-    def save_results_for_aggregator(
-        self, paths: af.DirectoryPaths, samples: af.Samples, model: af.Collection
-    ):
-        """
-        At the end of a model-fit,  this routine saves attributes of the `Analysis` object to the `pickles`
-        folder such that they can be loaded after the analysis using PyAutoFit's database and aggregator tools.
-
-        For this analysis it outputs the following:
-
-        - The stochastic log likelihoods of a pixelization, provided the pixelization has functionality that can
-        compute likelihoods for different KMeans seeds and grids (e.g. `VoronoiBrightnessImage).
-
-        Parameters
-        ----------
-        paths
-            The PyAutoFit paths object which manages all paths, e.g. where the non-linear search outputs are stored,
-            visualization, and the pickled objects used by the aggregator output by this function.
-        samples
-            A PyAutoFit object which contains the samples of the non-linear search, for example the chains of an MCMC
-            run of samples of the nested sampler.
-        model
-            The PyAutoFit model object, which includes model components representing the galaxies that are fitted to
-            the imaging data.
-        """
-        pixelization = ag.util.model.pixelization_from(model=model)
-
-        if conf.instance["general"]["hyper"]["stochastic_outputs"]:
-            if pixelization is not None:
-                if pixelization.is_stochastic:
-                    self.save_stochastic_outputs(paths=paths, samples=samples)
-
     def make_result(
-        self, samples: af.PDFSamples, model: af.Collection, search: af.NonLinearSearch
+        self,
+        samples: af.PDFSamples,
+        model: af.Collection,
+        sigma=1.0,
+        use_errors=True,
+        use_widths=False,
     ) -> ResultImaging:
         """
         After the non-linear search is complete create its `Result`, which includes:
@@ -517,15 +515,13 @@ class AnalysisImaging(AnalysisDataset):
         model
             The PyAutoFit model object, which includes model components representing the galaxies that are fitted to
             the imaging data.
-        search
-            The non-linear search used to perform this model-fit.
 
         Returns
         -------
         ResultImaging
             The result of fitting the model to the imaging dataset, via a non-linear search.
         """
-        return ResultImaging(samples=samples, model=model, analysis=self, search=search)
+        return ResultImaging(samples=samples, model=model, analysis=self)
 
     def save_attributes_for_aggregator(self, paths: af.DirectoryPaths):
         """
@@ -544,12 +540,12 @@ class AnalysisImaging(AnalysisDataset):
 
         This function also outputs attributes specific to an imaging dataset:
 
-       - Its PSF.
-       - Its mask.
-       - The positions of the brightest pixels in the lensed source which are used to discard mass models.
-       - The preloaded image-plane source plane pixelization if used by the analysis. This ensures that differences in
-       the scikit-learn library do not lead to different pixelizations being computed if results are transferred from
-       a HPC to laptop.
+        - Its PSF.
+        - Its mask.
+        - The positions of the brightest pixels in the lensed source which are used to discard mass models.
+        - The preloaded image-plane source plane pixelization if used by the analysis. This ensures that differences in
+        the scikit-learn library do not lead to different pixelizations being computed if results are transferred from
+        a HPC to laptop.
 
         It is common for these attributes to be loaded by many of the template aggregator functions given in the
         `aggregator` modules. For example, when using the database tools to perform a fit, the default behaviour is for
@@ -566,9 +562,4 @@ class AnalysisImaging(AnalysisDataset):
 
         paths.save_object("psf", self.dataset.psf_unormalized)
         paths.save_object("mask", self.dataset.mask)
-        paths.save_object("positions", self.positions)
-        if self.preloads.sparse_image_plane_grid_pg_list is not None:
-            paths.save_object(
-                "preload_sparse_grids_of_planes",
-                self.preloads.sparse_image_plane_grid_pg_list,
-            )
+        paths.save_object("positions_likelihood", self.positions_likelihood)
