@@ -6,19 +6,22 @@ from autoconf import cached_property
 import autoarray as aa
 import autogalaxy as ag
 
-from autolens.lens.model.preloads import Preloads
+from autogalaxy.abstract_fit import AbstractFit
+
+from autolens.analysis.preloads import Preloads
+from autolens.lens.ray_tracing import Tracer
 
 
-class FitInterferometer(aa.FitInterferometer):
+class FitInterferometer(aa.FitInterferometer, AbstractFit):
     def __init__(
         self,
-        dataset,
-        tracer,
-        hyper_background_noise=None,
-        use_hyper_scaling=True,
-        settings_pixelization=aa.SettingsPixelization(),
-        settings_inversion=aa.SettingsInversion(),
-        preloads=Preloads(),
+        dataset: aa.Interferometer,
+        tracer: Tracer,
+        hyper_background_noise: ag.hyper_data.HyperBackgroundNoise = None,
+        use_hyper_scaling: bool = True,
+        settings_pixelization: aa.SettingsPixelization = aa.SettingsPixelization(),
+        settings_inversion: aa.SettingsInversion = aa.SettingsInversion(),
+        preloads: Preloads = Preloads(),
         profiling_dict: Optional[Dict] = None,
     ):
         """
@@ -44,6 +47,9 @@ class FitInterferometer(aa.FitInterferometer):
         self.profiling_dict = profiling_dict
 
         super().__init__(dataset=dataset, profiling_dict=profiling_dict)
+        AbstractFit.__init__(
+            self=self, model_obj=tracer, settings_inversion=settings_inversion
+        )
 
     @property
     def noise_map(self):
@@ -65,7 +71,7 @@ class FitInterferometer(aa.FitInterferometer):
         Returns the visibilities of every light profile in the plane, which are computed by performing a Fourier
         transform to the sum of light profile images.
         """
-        return self.tracer.visibilities_via_transformer_from(
+        return self.tracer.visibilities_from(
             grid=self.dataset.grid, transformer=self.dataset.transformer
         )
 
@@ -85,19 +91,13 @@ class FitInterferometer(aa.FitInterferometer):
 
         The image passed to this function is the dataset's image with all light profile images of the plane subtracted.
         """
-        if self.tracer.has_pixelization:
+        if self.perform_inversion:
 
-            if self.settings_inversion.use_w_tilde:
-                w_tilde = self.dataset.w_tilde
-            else:
-                w_tilde = None
-
-            return self.tracer.inversion_interferometer_from(
-                grid=self.dataset.grid_inversion,
+            return self.tracer.to_inversion.inversion_interferometer_from(
+                dataset=self.dataset,
                 visibilities=self.profile_subtracted_visibilities,
                 noise_map=self.noise_map,
-                transformer=self.dataset.transformer,
-                w_tilde=w_tilde,
+                w_tilde=self.w_tilde,
                 settings_pixelization=self.settings_pixelization,
                 settings_inversion=self.settings_inversion,
                 preloads=self.preloads,
@@ -114,7 +114,9 @@ class FitInterferometer(aa.FitInterferometer):
         If a inversion is included it is the sum of this sum and the inversion's reconstruction of the image.
         """
 
-        if self.tracer.has_pixelization:
+        if self.tracer.has(cls=aa.pix.Pixelization) or self.tracer.has(
+            cls=ag.lp_linear.LightProfileLinear
+        ):
 
             return self.profile_visibilities + self.inversion.mapped_reconstructed_data
 
@@ -125,69 +127,48 @@ class FitInterferometer(aa.FitInterferometer):
         return self.interferometer.grid
 
     @property
-    def galaxy_model_image_dict(self) -> {ag.Galaxy: np.ndarray}:
+    def galaxy_model_image_dict(self) -> Dict[ag.Galaxy, np.ndarray]:
         """
         A dictionary associating galaxies with their corresponding model images
         """
         galaxy_model_image_dict = self.tracer.galaxy_image_2d_dict_from(grid=self.grid)
 
-        for path, image in galaxy_model_image_dict.items():
-            galaxy_model_image_dict[path] = image.binned
+        galaxy_linear_obj_image_dict = self.galaxy_linear_obj_data_dict_from(
+            use_image=True
+        )
 
-        # TODO : Extend to multiple inversioons across Planes
-
-        for plane_index in self.tracer.plane_indexes_with_pixelizations:
-
-            galaxy_model_image_dict.update(
-                {
-                    self.tracer.planes[plane_index].galaxies[
-                        0
-                    ]: self.inversion.mapped_reconstructed_image
-                }
-            )
-
-        return galaxy_model_image_dict
+        return {**galaxy_model_image_dict, **galaxy_linear_obj_image_dict}
 
     @property
-    def galaxy_model_visibilities_dict(self) -> {ag.Galaxy: np.ndarray}:
-        """
-        A dictionary associating galaxies with their corresponding model images
-        """
-        galaxy_model_visibilities_dict = self.tracer.galaxy_visibilities_dict_via_transformer_from(
+    def galaxy_model_visibilities_dict(self) -> Dict[ag.Galaxy, np.ndarray]:
+
+        galaxy_model_visibilities_dict = self.tracer.galaxy_visibilities_dict_from(
             grid=self.interferometer.grid, transformer=self.interferometer.transformer
         )
 
-        # TODO : Extend to multiple inversioons across Planes
-
-        for plane_index in self.tracer.plane_indexes_with_pixelizations:
-
-            galaxy_model_visibilities_dict.update(
-                {
-                    self.tracer.planes[plane_index].galaxies[
-                        0
-                    ]: self.inversion.mapped_reconstructed_data
-                }
-            )
-
-        return galaxy_model_visibilities_dict
-
-    def model_visibilities_of_planes(self):
-
-        model_visibilities_of_planes = self.tracer.visibilities_list_via_transformer_from(
-            grid=self.interferometer.grid, transformer=self.interferometer.transformer
+        galaxy_linear_obj_visibilities_dict = self.galaxy_linear_obj_data_dict_from(
+            use_image=False
         )
 
-        for plane_index in self.tracer.plane_indexes_with_pixelizations:
-
-            model_visibilities_of_planes[
-                plane_index
-            ] += self.inversion.mapped_reconstructed_image
-
-        return model_visibilities_of_planes
+        return {**galaxy_model_visibilities_dict, **galaxy_linear_obj_visibilities_dict}
 
     @property
-    def total_mappers(self):
-        return len(list(filter(None, self.tracer.regularization_pg_list)))
+    def model_visibilities_of_planes_list(self):
+
+        galaxy_model_visibilities_dict = self.galaxy_model_visibilities_dict
+
+        model_visibilities_of_planes_list = [
+            aa.Visibilities.zeros(shape_slim=(self.dataset.visibilities.shape_slim,))
+            for i in range(self.tracer.total_planes)
+        ]
+
+        for plane_index, plane in enumerate(self.tracer.planes):
+            for galaxy in plane.galaxies:
+                model_visibilities_of_planes_list[
+                    plane_index
+                ] += galaxy_model_visibilities_dict[galaxy]
+
+        return model_visibilities_of_planes_list
 
     def refit_with_new_preloads(self, preloads, settings_inversion=None):
 
