@@ -1,22 +1,22 @@
-from astropy import cosmology as cosmo
 import logging
 import numpy as np
-from typing import Optional
+from typing import Optional, Union
 
-from autoconf import conf
 import autofit as af
 import autoarray as aa
 import autogalaxy as ag
 
 from autoarray.exc import PixelizationException
 
-from autolens.lens.model.analysis import AnalysisDataset
-from autolens.lens.model.preloads import Preloads
+from autolens.analysis.analysis import AnalysisDataset
+from autolens.analysis.preloads import Preloads
+from autolens.analysis.positions import PositionsLHResample
+from autolens.analysis.positions import PositionsLHPenalty
 from autolens.lens.ray_tracing import Tracer
 from autolens.interferometer.model.result import ResultInterferometer
 from autolens.interferometer.model.visualizer import VisualizerInterferometer
 from autolens.interferometer.fit_interferometer import FitInterferometer
-from autolens.lens.model.settings import SettingsLens
+from autolens.analysis.settings import SettingsLens
 
 from autolens import exc
 
@@ -29,12 +29,15 @@ class AnalysisInterferometer(AnalysisDataset):
     def __init__(
         self,
         dataset,
-        positions: aa.Grid2DIrregular = None,
+        positions_likelihood: Optional[
+            Union[PositionsLHResample, PositionsLHPenalty]
+        ] = None,
         hyper_dataset_result=None,
-        cosmology=cosmo.Planck15,
+        cosmology: ag.cosmo.LensingCosmology = ag.cosmo.Planck15(),
         settings_pixelization: aa.SettingsPixelization = None,
         settings_inversion: aa.SettingsInversion = None,
         settings_lens: SettingsLens = None,
+        raise_inversion_positions_likelihood_exception: bool = True,
     ):
         """
         Analysis classes are used by PyAutoFit to fit a model to a dataset via a non-linear search.
@@ -55,6 +58,10 @@ class AnalysisInterferometer(AnalysisDataset):
         ----------
         dataset
             The interferometer dataset that the model is fitted too.
+        positions_likelihood
+            An object which alters the likelihood function to include a term which accounts for whether
+            image-pixel coordinates in arc-seconds corresponding to the multiple images of the lensed source galaxy
+            trace close to one another in the source-plane.
         hyper_dataset_result
             The hyper-model image and hyper galaxies images of a previous result in a model-fitting pipeline, which are
             used by certain classes for adapting the analysis to the properties of the dataset.
@@ -68,15 +75,21 @@ class AnalysisInterferometer(AnalysisDataset):
         settings_lens
             Settings controlling the lens calculation, for example how close the lensed source's multiple images have
             to trace within one another in the source plane for the model to not be discarded.
+        raise_inversion_positions_likelihood_exception
+            If an inversion is used without the `positions_likelihood` it is likely a systematic solution will
+            be inferred, in which case an Exception is raised before the model-fit begins to inform the user
+            of this. This exception is not raised if this input is False, allowing the user to perform the model-fit
+            anyway.
         """
         super().__init__(
             dataset=dataset,
-            positions=positions,
+            positions_likelihood=positions_likelihood,
             hyper_dataset_result=hyper_dataset_result,
             cosmology=cosmology,
             settings_pixelization=settings_pixelization,
             settings_inversion=settings_inversion,
             settings_lens=settings_lens,
+            raise_inversion_positions_likelihood_exception=raise_inversion_positions_likelihood_exception,
         )
 
         if self.hyper_dataset_result is not None:
@@ -110,7 +123,7 @@ class AnalysisInterferometer(AnalysisDataset):
             The PyAutoFit model object, which includes model components representing the galaxies that are fitted to
             the imaging data.
         """
-        self.check_and_replace_hyper_images(paths=paths)
+        super().modify_before_fit(paths=paths, model=model)
 
         if not paths.is_complete:
 
@@ -118,6 +131,12 @@ class AnalysisInterferometer(AnalysisDataset):
 
             visualizer.visualize_interferometer(interferometer=self.interferometer)
 
+            if self.positions_likelihood is not None:
+                visualizer.visualize_image_with_positions(
+                    image=self.interferometer.dirty_image,
+                    positions=self.positions_likelihood.positions,
+                )
+
             visualizer.visualize_hyper_images(
                 hyper_galaxy_image_path_dict=self.hyper_galaxy_image_path_dict,
                 hyper_model_image=self.hyper_model_image,
@@ -235,6 +254,16 @@ class AnalysisInterferometer(AnalysisDataset):
         float
             The log likelihood indicating how well this model instance fitted the interferometer data.
         """
+
+        try:
+            log_likelihood_positions_overwrite = self.log_likelihood_positions_overwrite_from(
+                instance=instance
+            )
+            if log_likelihood_positions_overwrite is not None:
+                return log_likelihood_positions_overwrite
+        except Exception as e:
+            raise e
+
         try:
             return self.fit_interferometer_via_instance_from(
                 instance=instance
@@ -245,6 +274,7 @@ class AnalysisInterferometer(AnalysisDataset):
             exc.InversionException,
             exc.GridException,
             ValueError,
+            TypeError,
             np.linalg.LinAlgError,
             OverflowError,
         ) as e:
@@ -255,7 +285,6 @@ class AnalysisInterferometer(AnalysisDataset):
         instance: af.ModelInstance,
         use_hyper_scaling: bool = True,
         preload_overwrite: Optional[Preloads] = None,
-        check_positions: bool = True,
     ) -> FitInterferometer:
         """
         Given a model instance create a `FitInterferometer` object.
@@ -287,11 +316,6 @@ class AnalysisInterferometer(AnalysisDataset):
         self.instance_with_associated_hyper_images_from(instance=instance)
         tracer = self.tracer_via_instance_from(instance=instance)
 
-        if check_positions:
-            self.settings_lens.check_positions_trace_within_threshold_via_tracer(
-                tracer=tracer, positions=self.positions
-            )
-
         hyper_background_noise = self.hyper_background_noise_via_instance_from(
             instance=instance
         )
@@ -380,10 +404,12 @@ class AnalysisInterferometer(AnalysisDataset):
         instance = self.instance_with_associated_hyper_images_from(instance=instance)
         tracer = self.tracer_via_instance_from(instance=instance)
 
-        if not tracer.has_pixelization:
+        if not tracer.has(cls=aa.pix.Pixelization):
             return None
 
-        if not any([pix.is_stochastic for pix in tracer.pixelization_list]):
+        if not any(
+            [pix.is_stochastic for pix in tracer.cls_list_from(cls=ag.pix.Pixelization)]
+        ):
             return
 
         hyper_background_noise = self.hyper_background_noise_via_instance_from(
@@ -458,6 +484,17 @@ class AnalysisInterferometer(AnalysisDataset):
 
         fit = self.fit_interferometer_via_instance_from(instance=instance)
 
+        if self.positions_likelihood is not None:
+            self.positions_likelihood.output_positions_info(
+                output_path=paths.output_path, tracer=fit.tracer
+            )
+
+        if fit.inversion is not None:
+            try:
+                fit.inversion.reconstruction
+            except exc.InversionException:
+                return
+
         visualizer = VisualizerInterferometer(visualize_path=paths.image_path)
 
         visualizer.visualize_fit_interferometer(
@@ -470,9 +507,12 @@ class AnalysisInterferometer(AnalysisDataset):
             galaxies=fit.tracer.galaxies, grid=fit.grid, during_analysis=during_analysis
         )
         if fit.inversion is not None:
-            visualizer.visualize_inversion(
-                inversion=fit.inversion, during_analysis=during_analysis
-            )
+            try:
+                visualizer.visualize_inversion(
+                    inversion=fit.inversion, during_analysis=during_analysis
+                )
+            except IndexError:
+                pass
 
         visualizer.visualize_contribution_maps(tracer=fit.tracer)
 
@@ -488,39 +528,13 @@ class AnalysisInterferometer(AnalysisDataset):
                 fit=fit, during_analysis=during_analysis, subfolders="fit_no_hyper"
             )
 
-    def save_results_for_aggregator(
-        self, paths: af.DirectoryPaths, samples: af.Samples, model: af.Collection
-    ):
-        """
-        At the end of a model-fit,  this routine saves attributes of the `Analysis` object to the `pickles`
-        folder such that they can be loaded after the analysis using PyAutoFit's database and aggregator tools.
-
-        For this analysis it outputs the following:
-
-        - The stochastic log likelihoods of a pixelization, provided the pixelization has functionality that can
-        compute likelihoods for different KMeans seeds and grids (e.g. `VoronoiBrightnessImage).
-
-        Parameters
-        ----------
-        paths
-            The PyAutoFit paths object which manages all paths, e.g. where the non-linear search outputs are stored,
-            visualization, and the pickled objects used by the aggregator output by this function.
-        samples
-            A PyAutoFit object which contains the samples of the non-linear search, for example the chains of an MCMC
-            run of samples of the nested sampler.
-        model
-            The PyAutoFit model object, which includes model components representing the galaxies that are fitted to
-            the imaging data.
-        """
-        pixelization = ag.util.model.pixelization_from(model=model)
-
-        if conf.instance["general"]["hyper"]["stochastic_outputs"]:
-            if pixelization is not None:
-                if pixelization.is_stochastic:
-                    self.save_stochastic_outputs(paths=paths, samples=samples)
-
     def make_result(
-        self, samples: af.PDFSamples, model: af.Collection, search: af.NonLinearSearch
+        self,
+        samples: af.PDFSamples,
+        model: af.Collection,
+        sigma=1.0,
+        use_errors=True,
+        use_widths=False,
     ):
         """
         After the non-linear search is complete create its `Result`, which includes:
@@ -544,17 +558,13 @@ class AnalysisInterferometer(AnalysisDataset):
         model
             The PyAutoFit model object, which includes model components representing the galaxies that are fitted to
             the imaging data.
-        search
-            The non-linear search used to perform this model-fit.
 
         Returns
         -------
         ResultImaging
             The result of fitting the model to the imaging dataset, via a non-linear search.
         """
-        return ResultInterferometer(
-            samples=samples, model=model, analysis=self, search=search
-        )
+        return ResultInterferometer(samples=samples, model=model, analysis=self)
 
     def save_attributes_for_aggregator(self, paths: af.DirectoryPaths):
         """
@@ -595,7 +605,7 @@ class AnalysisInterferometer(AnalysisDataset):
 
         paths.save_object("uv_wavelengths", self.dataset.uv_wavelengths)
         paths.save_object("real_space_mask", self.dataset.real_space_mask)
-        paths.save_object("positions", self.positions)
+        paths.save_object("positions_likelihood", self.positions_likelihood)
         if self.preloads.sparse_image_plane_grid_pg_list is not None:
             paths.save_object(
                 "preload_sparse_grids_of_planes",
