@@ -1,18 +1,24 @@
+from autolens.lens.mock.mock_to_inversion import MockTracerToInversion
+
+
 class MockTracer:
     def __init__(
-        self, traced_grids_of_planes=None, sparse_image_plane_grid_pg_list=None
+        self, traced_grid_2d_list_from=None, sparse_image_plane_grid_pg_list=None
     ):
 
-        self.traced_grids_of_planes = traced_grids_of_planes
         self.sparse_image_plane_grid_pg_list = sparse_image_plane_grid_pg_list
+        self._traced_grid_2d_list_from = traced_grid_2d_list_from
 
-    def traced_grid_list_from(self, grid):
+    def traced_grid_2d_list_from(self, grid):
 
-        return self.traced_grids_of_planes
+        return self._traced_grid_2d_list_from
 
-    def sparse_image_plane_grid_pg_list_from(self, grid):
-
-        return self.sparse_image_plane_grid_pg_list
+    @property
+    def to_inversion(self):
+        return MockTracerToInversion(
+            tracer=MockTracer,
+            sparse_image_plane_grid_pg_list=self.sparse_image_plane_grid_pg_list,
+        )
 
 
 class MockTracerPoint(MockTracer):
@@ -47,17 +53,13 @@ class MockTracerPoint(MockTracer):
     def deflections_yx_2d_from(self):
         pass
 
-    @property
-    def has_mass_profile(self):
-        return True
-
     def extract_attribute(self, cls, attr_name):
         return [self.attribute]
 
     def extract_profile(self, profile_name):
         return self.profile
 
-    def traced_grid_list_from(self, grid, plane_index_limit=None):
+    def traced_grid_2d_list_from(self, grid, plane_index_limit=None):
         return [self.positions]
 
     def magnification_2d_via_hessian_from(self, grid, deflections_func=None):
