@@ -1,27 +1,27 @@
 from abc import ABC
-from astropy import cosmology as cosmo
-import json
 import numpy as np
-from typing import Dict, List, Optional, Tuple, Union
+from typing import Dict, List, Optional, Type, Union
 
 import autoarray as aa
 import autogalaxy as ag
 
 from autoconf.dictable import Dictable
 
-from autoarray.inversion.inversion.factory import inversion_imaging_unpacked_from
-from autoarray.inversion.inversion.factory import inversion_interferometer_unpacked_from
-
 from autogalaxy.plane.plane import Plane
 from autogalaxy.profiles.light_profiles.light_profiles_snr import LightProfileSNR
 
-from autolens.lens.model.preloads import Preloads
+from autolens.lens.to_inversion import TracerToInversion
 
 from autolens.lens import ray_tracing_util
 
 
 class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
-    def __init__(self, planes, cosmology, profiling_dict: Optional[Dict] = None):
+    def __init__(
+        self,
+        planes,
+        cosmology: ag.cosmo.LensingCosmology,
+        profiling_dict: Optional[Dict] = None,
+    ):
         """
         Ray-tracer for a lens system with any number of planes.
 
@@ -54,7 +54,10 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
 
     @classmethod
     def from_galaxies(
-        cls, galaxies, cosmology=cosmo.Planck15, profiling_dict: Optional[Dict] = None
+        cls,
+        galaxies,
+        cosmology: ag.cosmo.LensingCosmology = ag.cosmo.Planck15(),
+        profiling_dict: Optional[Dict] = None,
     ):
 
         planes = ag.util.plane.planes_via_galaxies_from(
@@ -70,7 +73,7 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
         line_of_sight_galaxies,
         source_galaxies,
         planes_between_lenses,
-        cosmology=cosmo.Planck15,
+        cosmology: ag.cosmo.LensingCosmology = ag.cosmo.Planck15(),
     ):
 
         """Ray-tracer for a lens system with any number of planes.
@@ -138,7 +141,7 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
 
     @staticmethod
     def from_dict(cls_dict):
-        cls_dict["cosmology"] = getattr(cosmo, cls_dict["cosmology"])
+        cls_dict["cosmology"] = getattr(ag.cosmo, cls_dict["cosmology"])
         cls_dict["planes"] = list(map(Plane.from_dict, cls_dict["planes"]))
         return Dictable.from_dict(cls_dict)
 
@@ -146,6 +149,30 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
     def galaxies(self) -> List[ag.Galaxy]:
         return list([galaxy for plane in self.planes for galaxy in plane.galaxies])
 
+    def has(self, cls: Type) -> bool:
+        return any(map(lambda plane: plane.has(cls=cls), self.planes))
+
+    def cls_list_from(self, cls: Type) -> List:
+        """
+        Returns a list of objects in the tracer which are an instance of the input `cls`.
+
+        For example:
+
+        - If the input is `cls=ag.lp.LightProfile`, a list containing all light profiles in the tracer is returned.
+
+        Returns
+        -------
+            The list of objects in the tracer that inherit from input `cls`.
+        """
+        cls_list = []
+
+        for galaxy in self.galaxies:
+            if galaxy.has(cls=cls):
+                for cls_galaxy in galaxy.cls_list_from(cls=cls):
+                    cls_list.append(cls_galaxy)
+
+        return cls_list
+
     @property
     def total_planes(self) -> int:
         return len(self.plane_redshifts)
@@ -165,35 +192,15 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
     def plane_with_galaxy(self, galaxy) -> Plane:
         return [plane for plane in self.planes if galaxy in plane.galaxies][0]
 
-    @aa.profile_func
-    def sparse_image_plane_grid_pg_list_from(
-        self, grid: aa.type.Grid2DLike, settings_pixelization=aa.SettingsPixelization()
-    ) -> List[List]:
-        """
-        Specific pixelizations, like the `VoronoiMagnification`, begin by determining what will become its the
-        source-pixel centres by calculating them  in the image-plane. The `VoronoiBrightnessImage` pixelization
-        performs a KMeans clustering.
-        """
-
-        sparse_image_plane_grid_list_of_planes = []
-
-        for plane in self.planes:
-            sparse_image_plane_grid_list = plane.sparse_image_plane_grid_list_from(
-                grid=grid, settings_pixelization=settings_pixelization
-            )
-            sparse_image_plane_grid_list_of_planes.append(sparse_image_plane_grid_list)
-
-        return sparse_image_plane_grid_list_of_planes
-
     @aa.grid_dec.grid_2d_to_structure_list
-    def traced_grid_list_from(
+    def traced_grid_2d_list_from(
         self, grid: aa.type.Grid2DLike, plane_index_limit=None
     ) -> List[aa.type.Grid2DLike]:
         """
         Performs multi-plane ray tracing on a 2D grid of Cartesian (y,x) coordinates using the mass profiles of the
         galaxies and planes contained within the tracer.
 
-        see `autolens.lens.ray_tracing.ray_tracing_util.traced_grid_list_from()` for a full description of the
+        see `autolens.lens.ray_tracing.ray_tracing_util.traced_grid_2d_list_from()` for a full description of the
         calculation.
 
         Parameters
@@ -210,66 +217,14 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
             A list of 2D (y,x) grids each of which are the input grid ray-traced to a redshift of the input list of planes.
         """
 
-        return ray_tracing_util.traced_grid_list_from(
+        return ray_tracing_util.traced_grid_2d_list_from(
             planes=self.planes,
             grid=grid,
             cosmology=self.cosmology,
             plane_index_limit=plane_index_limit,
         )
 
-    @aa.profile_func
-    def traced_sparse_grid_pg_list_from(
-        self,
-        grid: aa.type.Grid2DLike,
-        settings_pixelization=aa.SettingsPixelization(),
-        preloads=Preloads(),
-    ) -> Tuple[List[List], List[List]]:
-        """
-        Ray-trace the sparse image plane grid used to define the source-pixel centres by calculating the deflection
-        angles at (y,x) coordinate on the grid from the galaxy mass profiles and then ray-trace them from the
-        image-plane to the source plane.
-        """
-        if (
-            preloads.sparse_image_plane_grid_pg_list is None
-            or settings_pixelization.is_stochastic
-        ):
-
-            sparse_image_plane_grid_pg_list = self.sparse_image_plane_grid_pg_list_from(
-                grid=grid, settings_pixelization=settings_pixelization
-            )
-
-        else:
-
-            sparse_image_plane_grid_pg_list = preloads.sparse_image_plane_grid_pg_list
-
-        traced_sparse_grid_pg_list = []
-
-        for (plane_index, plane) in enumerate(self.planes):
-
-            if sparse_image_plane_grid_pg_list[plane_index] is None:
-                traced_sparse_grid_pg_list.append(None)
-            else:
-
-                traced_sparse_grids_list = []
-
-                for sparse_image_plane_grid in sparse_image_plane_grid_pg_list[
-                    plane_index
-                ]:
-
-                    try:
-                        traced_sparse_grids_list.append(
-                            self.traced_grid_list_from(grid=sparse_image_plane_grid)[
-                                plane_index
-                            ]
-                        )
-                    except AttributeError:
-                        traced_sparse_grids_list.append(None)
-
-                traced_sparse_grid_pg_list.append(traced_sparse_grids_list)
-
-        return traced_sparse_grid_pg_list, sparse_image_plane_grid_pg_list
-
-    def grid_at_redshift_from(
+    def grid_2d_at_redshift_from(
         self, grid: aa.type.Grid2DLike, redshift: float
     ) -> aa.type.Grid2DLike:
         """
@@ -287,31 +242,34 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
         redshift
             The redshift the image-plane grid is traced to.
         """
-        return ray_tracing_util.grid_at_redshift_from(
+        return ray_tracing_util.grid_2d_at_redshift_from(
             redshift=redshift,
             galaxies=self.galaxies,
             grid=grid,
             cosmology=self.cosmology,
         )
 
-    @property
-    def has_light_profile(self) -> bool:
-        return any(list(map(lambda plane: plane.has_light_profile, self.planes)))
-
     @aa.grid_dec.grid_2d_to_structure
     @aa.profile_func
-    def image_2d_from(self, grid: aa.type.Grid2DLike) -> aa.Array2D:
-        return sum(self.image_2d_list_from(grid=grid))
+    def image_2d_from(
+        self, grid: aa.type.Grid2DLike, operated_only: Optional[bool] = None
+    ) -> aa.Array2D:
+
+        return sum(self.image_2d_list_from(grid=grid, operated_only=operated_only))
 
     @aa.grid_dec.grid_2d_to_structure_list
-    def image_2d_list_from(self, grid: aa.type.Grid2DLike) -> List[aa.Array2D]:
+    def image_2d_list_from(
+        self, grid: aa.type.Grid2DLike, operated_only: Optional[bool] = None
+    ) -> List[aa.Array2D]:
 
-        traced_grid_list = self.traced_grid_list_from(
+        traced_grid_list = self.traced_grid_2d_list_from(
             grid=grid, plane_index_limit=self.upper_plane_index_with_light_profile
         )
 
         image_2d_list = [
-            self.planes[plane_index].image_2d_from(grid=traced_grid_list[plane_index])
+            self.planes[plane_index].image_2d_from(
+                grid=traced_grid_list[plane_index], operated_only=operated_only
+            )
             for plane_index in range(len(traced_grid_list))
         ]
 
@@ -324,8 +282,8 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
         return image_2d_list
 
     def galaxy_image_2d_dict_from(
-        self, grid: aa.type.Grid2DLike
-    ) -> {ag.Galaxy: np.ndarray}:
+        self, grid: aa.type.Grid2DLike, operated_only: Optional[bool] = None
+    ) -> Dict[ag.Galaxy, np.ndarray]:
         """
         Returns a dictionary associating every `Galaxy` object in the `Tracer` with its corresponding 2D image, using
         the instance of each galaxy as the dictionary keys.
@@ -348,22 +306,20 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
         A dictionary associated every galaxy in the tracer with its corresponding 2D image.
         """
 
-        galaxy_image_dict = dict()
+        galaxy_image_2d_dict = dict()
 
-        traced_grid_list = self.traced_grid_list_from(grid=grid)
+        traced_grid_list = self.traced_grid_2d_list_from(grid=grid)
 
         for (plane_index, plane) in enumerate(self.planes):
-            images_of_galaxies = plane.image_2d_list_from(
-                grid=traced_grid_list[plane_index]
+            image_2d_list = plane.image_2d_list_from(
+                grid=traced_grid_list[plane_index], operated_only=operated_only
             )
+
             for (galaxy_index, galaxy) in enumerate(plane.galaxies):
-                galaxy_image_dict[galaxy] = images_of_galaxies[galaxy_index]
 
-        return galaxy_image_dict
+                galaxy_image_2d_dict[galaxy] = image_2d_list[galaxy_index]
 
-    @property
-    def has_mass_profile(self) -> bool:
-        return any(list(map(lambda plane: plane.has_mass_profile, self.planes)))
+        return galaxy_image_2d_dict
 
     @aa.grid_dec.grid_2d_to_vector_yx
     @aa.grid_dec.grid_2d_to_structure
@@ -387,7 +343,7 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
         self, grid: aa.type.Grid2DLike, plane_i=0, plane_j=-1
     ) -> Union[aa.VectorYX2D, aa.VectorYX2DIrregular]:
 
-        traced_grids_list = self.traced_grid_list_from(grid=grid)
+        traced_grids_list = self.traced_grid_2d_list_from(grid=grid)
 
         return traced_grids_list[plane_i] - traced_grids_list[plane_j]
 
@@ -399,43 +355,19 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
     def potential_2d_from(self, grid: aa.type.Grid2DLike) -> aa.Array2D:
         return sum([plane.potential_2d_from(grid=grid) for plane in self.planes])
 
-    @property
-    def has_pixelization(self) -> bool:
-        return any(list(map(lambda plane: plane.has_pixelization, self.planes)))
-
-    @property
-    def has_regularization(self) -> bool:
-        return any(list(map(lambda plane: plane.has_regularization, self.planes)))
-
-    @aa.profile_func
-    def traced_grid_list_of_inversion_from(
-        self, grid: aa.type.Grid2DLike
-    ) -> List[aa.type.Grid2DLike]:
-        return self.traced_grid_list_from(grid=grid)
-
-    @property
-    def has_hyper_galaxy(self) -> bool:
-        return any(list(map(lambda plane: plane.has_hyper_galaxy, self.planes)))
-
     @property
     def upper_plane_index_with_light_profile(self) -> int:
         return max(
             [
-                plane_index if plane.has_light_profile else 0
+                plane_index if plane.has(cls=ag.lp.LightProfile) else 0
                 for (plane_index, plane) in enumerate(self.planes)
             ]
         )
 
-    @property
-    def mass_profile_list(self):
-        return [
-            plane.mass_profile_list for plane in self.planes if plane.has_mass_profile
-        ]
-
     @property
     def plane_indexes_with_pixelizations(self):
         plane_indexes_with_inversions = [
-            plane_index if plane.has_pixelization else None
+            plane_index if plane.has(cls=aa.pix.Pixelization) else None
             for (plane_index, plane) in enumerate(self.planes)
         ]
         return [
@@ -444,160 +376,6 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
             if plane_index is not None
         ]
 
-    @property
-    def pixelization_list(self) -> List:
-        return [
-            galaxy.pixelization for galaxy in self.galaxies if galaxy.has_pixelization
-        ]
-
-    @property
-    def pixelization_pg_list(self) -> List[List]:
-        return [plane.pixelization_list for plane in self.planes]
-
-    @property
-    def regularization_list(self) -> List:
-        return [
-            galaxy.regularization
-            for galaxy in self.galaxies
-            if galaxy.has_regularization
-        ]
-
-    @property
-    def regularization_pg_list(self) -> List[List]:
-        return [plane.regularization_list for plane in self.planes]
-
-    def linear_obj_list_from(
-        self,
-        grid: aa.type.Grid2DLike,
-        settings_pixelization=aa.SettingsPixelization(),
-        preloads=Preloads(),
-    ):
-
-        linear_obj_list = []
-
-        if preloads.traced_grids_of_planes_for_inversion is None:
-            traced_grids_of_planes = self.traced_grid_list_of_inversion_from(grid=grid)
-        else:
-            traced_grids_of_planes = preloads.traced_grids_of_planes_for_inversion
-
-        if preloads.traced_sparse_grids_list_of_planes is None:
-            traced_sparse_grids_list_of_planes, sparse_image_plane_grid_list = self.traced_sparse_grid_pg_list_from(
-                grid=grid,
-                settings_pixelization=settings_pixelization,
-                preloads=preloads,
-            )
-        else:
-            traced_sparse_grids_list_of_planes = (
-                preloads.traced_sparse_grids_list_of_planes
-            )
-            sparse_image_plane_grid_list = preloads.sparse_image_plane_grid_list
-
-        for (plane_index, plane) in enumerate(self.planes):
-
-            if plane.has_pixelization:
-
-                for mapper_index in range(
-                    len(traced_sparse_grids_list_of_planes[plane_index])
-                ):
-
-                    mapper = plane.mapper_from(
-                        source_grid_slim=traced_grids_of_planes[plane_index],
-                        source_pixelization_grid=traced_sparse_grids_list_of_planes[
-                            plane_index
-                        ][mapper_index],
-                        data_pixelization_grid=sparse_image_plane_grid_list[
-                            plane_index
-                        ][mapper_index],
-                        pixelization=self.pixelization_pg_list[plane_index][
-                            mapper_index
-                        ],
-                        hyper_galaxy_image=self.hyper_galaxy_image_pg_list[plane_index][
-                            mapper_index
-                        ],
-                        settings_pixelization=settings_pixelization,
-                        preloads=preloads,
-                    )
-                    linear_obj_list.append(mapper)
-
-        return linear_obj_list
-
-    def inversion_imaging_from(
-        self,
-        grid,
-        image,
-        noise_map,
-        convolver,
-        w_tilde,
-        settings_pixelization=aa.SettingsPixelization(),
-        settings_inversion=aa.SettingsInversion(),
-        preloads=Preloads(),
-    ):
-
-        if preloads.linear_obj_list is None:
-
-            linear_obj_list = self.linear_obj_list_from(
-                grid=grid,
-                settings_pixelization=settings_pixelization,
-                preloads=preloads,
-            )
-
-        else:
-
-            linear_obj_list = preloads.linear_obj_list
-
-        return inversion_imaging_unpacked_from(
-            image=image,
-            noise_map=noise_map,
-            convolver=convolver,
-            w_tilde=w_tilde,
-            linear_obj_list=linear_obj_list,
-            regularization_list=self.regularization_list,
-            settings=settings_inversion,
-            preloads=preloads,
-            profiling_dict=self.profiling_dict,
-        )
-
-    def inversion_interferometer_from(
-        self,
-        grid,
-        visibilities,
-        noise_map,
-        transformer,
-        w_tilde,
-        settings_pixelization=aa.SettingsPixelization(),
-        settings_inversion=aa.SettingsInversion(),
-        preloads=Preloads(),
-    ):
-
-        if preloads.linear_obj_list is None:
-
-            linear_obj_list = self.linear_obj_list_from(
-                grid=grid,
-                settings_pixelization=settings_pixelization,
-                preloads=preloads,
-            )
-
-        else:
-
-            linear_obj_list = preloads.linear_obj_list
-
-        return inversion_interferometer_unpacked_from(
-            visibilities=visibilities,
-            noise_map=noise_map,
-            transformer=transformer,
-            w_tilde=w_tilde,
-            linear_obj_list=linear_obj_list,
-            regularization_list=self.regularization_list,
-            settings=settings_inversion,
-            profiling_dict=self.profiling_dict,
-        )
-
-    @property
-    def hyper_galaxy_image_pg_list(self) -> List[List]:
-        return [
-            plane.hyper_galaxies_with_pixelization_image_list for plane in self.planes
-        ]
-
     def hyper_noise_map_from(self, noise_map: aa.Array2D) -> aa.Array2D:
         return sum(self.hyper_noise_map_list_from(noise_map=noise_map))
 
@@ -615,8 +393,6 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
 
         if contribution_map_list:
             return sum(contribution_map_list)
-        else:
-            return None
 
     @property
     def contribution_map_list(self) -> List[aa.Array2D]:
@@ -635,6 +411,10 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
 
         return contribution_map_list
 
+    @property
+    def to_inversion(self):
+        return TracerToInversion(tracer=self)
+
     def extract_attribute(self, cls, attr_name):
         """
         Returns an attribute of a class in the tracer as a `ValueIrregular` or `Grid2DIrregular` object.
@@ -828,14 +608,14 @@ class Tracer(ABC, ag.OperateImageGalaxies, ag.OperateDeflections, Dictable):
             shape_native=grid.shape_native, pixel_scales=grid.pixel_scales, sub_size=1
         )
 
-        traced_grids_of_planes = self.traced_grid_list_from(grid=grid)
+        traced_grids_of_planes_list = self.traced_grid_2d_list_from(grid=grid)
 
         for plane_index, plane in enumerate(self.planes):
             for galaxy in plane.galaxies:
-                for light_profile in galaxy.light_profile_list:
+                for light_profile in galaxy.cls_list_from(cls=ag.lp.LightProfile):
                     if isinstance(light_profile, LightProfileSNR):
                         light_profile.set_intensity_from(
-                            grid=traced_grids_of_planes[plane_index],
+                            grid=traced_grids_of_planes_list[plane_index],
                             exposure_time=exposure_time,
                             background_sky_level=background_sky_level,
                         )
