@@ -1,4 +1,5 @@
 import datetime
+from typing import Any, Dict
 
 # Configuration file for the Sphinx documentation builder.
 #
@@ -12,6 +13,7 @@ import datetime
 # add these directories to sys.path here. If the directory is relative to the
 # documentation root, use os.path.abspath to make it absolute, like shown here.
 #
+
 from pyprojroot import here
 
 workspace_path = str(here())
@@ -26,91 +28,108 @@ sys.path.insert(0, os.path.abspath("."))
 
 year = datetime.date.today().year
 project = "PyAutoLens"
-copyright = "2020, James Nightingale, Richard Hayes"
+copyright = "2022, James Nightingale, Richard Hayes"
 author = "James Nightingale, Richard Hayes"
 
 # The full version, including alpha/beta/rc tags
-release = "2022.03.18.2"
+release = "2022.05.02.1"
 master_doc = "index"
 
 
 # -- General configuration ---------------------------------------------------
 
-# Add any Sphinx extension module names here, as strings. They can be
-# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
-# ones.
 extensions = [
     "sphinx.ext.autodoc",
     "sphinx.ext.autosummary",
-    "sphinx.ext.napoleon",
     "sphinx.ext.extlinks",
+    "sphinx.ext.intersphinx",
+    "sphinx.ext.mathjax",
+    "sphinx.ext.todo",
+    "sphinx.ext.napoleon",
+    "sphinx.ext.viewcode",
     "numpydoc",
+    # External stuff
+    "myst_parser",
+    "sphinx_copybutton",
+    "sphinx_design",
+    "sphinx_inline_tabs",
 ]
+templates_path = ["_templates"]
+
+# -- Options for extlinks ----------------------------------------------------
+
+extlinks = {"pypi": ("https://pypi.org/project/%s/", "")}
+
+# -- Options for intersphinx -------------------------------------------------
+
+intersphinx_mapping = {
+    "python": ("https://docs.python.org/3", None),
+    "sphinx": ("https://www.sphinx-doc.org/en/master", None),
+}
+
+# -- Options for TODOs -------------------------------------------------------
+
+todo_include_todos = True
+
+# -- Options for Markdown files ----------------------------------------------
+
+myst_enable_extensions = ["colon_fence", "deflist"]
+myst_heading_anchors = 3
 
-## Generate autodoc stubs with summaries from code
 autosummary_generate = True
-
-## Include Python objects as they appear in source files
 autodoc_member_order = "bysource"
-
-## Default flags used by autodoc directives
 autodoc_default_flags = ["members"]
 
 numpydoc_show_class_members = False
 numpydoc_show_inherited_class_members = False
 numpydoc_class_members_toctree = False
 
-# Add any paths that contain templates here, relative to this directory.
-templates_path = ["_templates"]
-
 # List of patterns, relative to source directory, that match files and
 # directories to ignore when looking for source files.
 # This pattern also affects html_static_path and html_extra_path.
 exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]
 
 
-sphinx_gallery_conf = {
-    # Remove the "Download all examples" button from the top level gallery
-    "download_all_examples": False,
-    # directory where function granular galleries are stored
-    "backreferences_dir": "api/generated/backreferences",
-    # Modules for which function level galleries are created.
-    "doc_module": "pyautolens",
-    # Insert links to documentation of objects in the examples
-    "reference_url": {"pyautolens": None},
-}
-
 # -- Options for HTML output -------------------------------------------------
 
-# The theme to use for HTML and HTML Help pages.  See the documentation for
-# a list of builtin themes.
-#
-html_theme = "sphinx_rtd_theme"
-html_last_updated_fmt = "%b %d, %Y"
+html_theme = "furo"
 html_title = "PyAutoLens"
 html_short_title = "PyAutoLens"
-pygments_style = "default"
-add_function_parentheses = False
+html_permalinks_icon = "<span>#</span>"
+html_last_updated_fmt = "%b %d, %Y"
+
 html_show_sourcelink = False
 html_show_sphinx = True
 html_show_copyright = True
 
+pygments_style = "sphinx"
+pygments_dark_style = "monokai"
+add_function_parentheses = False
+
 html_context = {
     "menu_links_name": "Repository",
-    # Custom variables to enable "Improve this page"" and "Download notebook"
-    # links
     "doc_path": "docs",
     "github_project": "pyautolens",
     "github_repo": "pyautolens",
-    "github_version": "development",
+    "github_version": "master",
+}
+language = "en"
+
+html_static_path = ["_static"]
+html_css_files = ["pied-piper-admonition.css"]
+
+html_theme_options = {
+    "light_css_variables": {
+        "color-brand-primary": "#7C4DFF",
+        "color-brand-content": "#7C4DFF",
+    }
 }
 
-# Add any paths that contain custom static files (such as style sheets) here,
-# relative to this directory. They are copied after the builtin static files,
-# so a file named "default.css" will overwrite the builtin "default.css".
-html_static_path = ["_static"]
-
-# -- IMAGES --
+if "READTHEDOCS" in os.environ:
+    html_theme_options["announcement"] = (
+        "This documentation is hosted on Read the Docs only for testing. Please use "
+        "<a href='https://pradyunsg.me/furo/'>the main documentation</a> instead."
+    )
 
 
 from sphinx.builders.html import StandaloneHTMLBuilder
